# -*- coding: utf-8 -*-
"""
Created on Sun Jan 20 18:14:02 2013

@author: dave
"""
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
from builtins import object




import numpy as np
import scipy as sp

from wetb.prepost import DataChecks as chk
from wetb.prepost.misc import calc_sample_rate
from wetb.prepost import mplutils


class Filters(object):

    def __init__(self):
        pass


    def smooth(self, x, window_len=11, window='hanning'):
        """
        Smooth the data using a window with requested size
        ==================================================

        This method is based on the convolution of a scaled window with the
        signal. The signal is prepared by introducing reflected copies of the
        signal (with the window size) in both ends so that transient parts are
        minimized in the begining and end part of the output signal.

        input:
            x: the input signal
            window_len: the dimension of the smoothing window; should be an odd
            integer
            window: the type of window from 'flat', 'hanning', 'hamming',
            'bartlett', 'blackman' flat window will produce a moving average
            smoothing.

        output:
            the smoothed signal

        example:

        t=linspace(-2,2,0.1)
        x=sin(t)+randn(len(t))*0.1
        y=smooth(x)

        see also:

        numpy.hanning, numpy.hamming, numpy.bartlett, numpy.blackman,
        numpy.convolve, scipy.signal.lfilter

        TODO: the window parameter could be the window itself if an array
        instead of a string

        SOURCE: http://www.scipy.org/Cookbook/SignalSmooth
        """

        if x.ndim != 1:
            raise ValueError("smooth only accepts 1 dimension arrays.")

        if x.size < window_len:
            msg = "Input vector needs to be bigger than window size."
            raise ValueError(msg)

        if window_len<3:
            return x

        windowlist = ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']
        if not window in windowlist:
            msg = "Window should be 'flat', 'hanning', 'hamming', 'bartlett',"
            msg += " or 'blackman'"
            raise ValueError(msg)

        s = np.r_[x[window_len-1:0:-1],x,x[-1:-window_len:-1]]
        #print(len(s))
        if window == 'flat': #moving average
            w = np.ones(window_len,'d')
        else:
            w = eval('np.'+window+'(window_len)')

        y = np.convolve(w/w.sum(), s, mode='valid')
        return y

    def butter(self, time, data, **kwargs):
        """
        Source:
        https://azitech.wordpress.com/2011/03/15/
        designing-a-butterworth-low-pass-filter-with-scipy/
        """

        sample_rate = kwargs.get('sample_rate', None)
        if not sample_rate:
            sample_rate = calc_sample_rate(time)

        # The cutoff frequency of the filter.
        cutoff_hz = kwargs.get('cutoff_hz', 1.0)

        # design filter
        norm_pass = cutoff_hz/(sample_rate/2.0)
        norm_stop = 1.5*norm_pass
        (N, Wn) = sp.signal.buttord(wp=norm_pass, ws=norm_stop, gpass=2,
                                    gstop=30, analog=0)
        (b, a) = sp.signal.butter(N, Wn, btype='low', analog=0, output='ba')

        # filtered output
        #zi = signal.lfiltic(b, a, x[0:5], x[0:5])
        #(y, zi) = signal.lfilter(b, a, x, zi=zi)
        data_filt = sp.signal.lfilter(b, a, data)

        return data_filt

    def fir(self, time, data, **kwargs):
        """
        Based on the xxample from the SciPy cook boock, see
        http://www.scipy.org/Cookbook/FIRFilter

        Parameters
        ----------

        time : ndarray(n)

        data : ndarray(n)

        plot : boolean, default=False

        figpath : str, default=False

        figfile : str, default=False

        sample_rate : int, default=None
            If None, sample rate will be calculated from the given signal

        freq_trans_width : float, default=1
            The desired width of the transition from pass to stop,
            relative to the Nyquist rate.

        ripple_db : float, default=10
            The desired attenuation in the stop band, in dB.

        cutoff_hz : float, default=10
            Frequencies above cutoff_hz are filtered out

        Returns
        -------

        filtered_x : ndarray(n - (N-1))
            filtered signal

        N : float
            order of the firwin filter

        delay : float
            phase delay due to the filtering process

        """

        plot = kwargs.get('plot', False)
        figpath = kwargs.get('figpath', False)
        figfile = kwargs.get('figfile', False)

        sample_rate = kwargs.get('sample_rate', None)
        # The desired width of the transition from pass to stop,
        # relative to the Nyquist rate.  We'll design the filter
        # with a 5 Hz transition width.
        freq_trans_width = kwargs.get('freq_trans_width', 1)

        # The desired attenuation in the stop band, in dB.
        ripple_db = kwargs.get('ripple_db', 10)

        # The cutoff frequency of the filter.
        cutoff_hz = kwargs.get('cutoff_hz', 10)

        chk.array_1d(time)
        chk.array_1d(data)

        if not sample_rate:
            sample_rate = calc_sample_rate(time)

        #------------------------------------------------
        # Create a FIR filter and apply it to data[:,channel]
        #------------------------------------------------

        # The Nyquist rate of the signal.
        nyq_rate = sample_rate / 2.0

        # The desired width of the transition from pass to stop,
        # relative to the Nyquist rate.  We'll design the filter
        # with a 5 Hz transition width.
        width = freq_trans_width/nyq_rate

        # Compute the order and Kaiser parameter for the FIR filter.
        N, beta = sp.signal.kaiserord(ripple_db, width)

        # Use firwin with a Kaiser window to create a lowpass FIR filter.
        taps = sp.signal.firwin(N, cutoff_hz/nyq_rate,
                                  window=('kaiser', beta))

        # Use lfilter to filter x with the FIR filter.
        filtered_x = sp.signal.lfilter(taps, 1.0, data)

        # The phase delay of the filtered signal.
        delay = 0.5 * (N-1) / sample_rate

#        # the filtered signal, shifted to compensate for the phase delay.
#        time_shifted = time-delay
#        # the "good" part of the filtered signal.  The first N-1
#        # samples are "corrupted" by the initial conditions.
#        time_good = time[N-1:] - delay

        if plot:
            self.plot_fir(figpath, figfile, time, data, filtered_x, N, delay,
                 sample_rate, taps, nyq_rate)

        return filtered_x, N, delay


    def plot_fir(self, figpath, figfile, time, data, filtered_x, N, delay,
                 sample_rate, taps, nyq_rate):
        """
        """

        #------------------------------------------------
        # Setup the figure parameters
        #------------------------------------------------

        plot = mplutils.A4Tuned()
        plot.setup(figpath+figfile, nr_plots=3, grandtitle=figfile,
                         figsize_y=20, wsleft_cm=2.)

        #------------------------------------------------
        # Plot the FIR filter coefficients.
        #------------------------------------------------
        plot_nr = 1
        ax1 = plot.fig.add_subplot(plot.nr_rows, plot.nr_cols, plot_nr)
        ax1.plot(taps, 'bo-', linewidth=2)
        ax1.set_title('Filter Coefficients (%d taps)' % N)
        ax1.grid(True)

        #------------------------------------------------
        # Plot the magnitude response of the filter.
        #------------------------------------------------

        plot_nr += 1
        ax2 = plot.fig.add_subplot(plot.nr_rows, plot.nr_cols, plot_nr)

        w, h = sp.signal.freqz(taps, worN=8000)
        ax2.plot((w/np.pi)*nyq_rate, np.absolute(h), linewidth=2)
        ax2.set_xlabel('Frequency (Hz)')
        ax2.set_ylabel('Gain')
        ax2.set_title('Frequency Response')
        ax2.set_ylim(-0.05, 1.05)
#        ax2.grid(True)

        # in order to place the nex axes inside following figure, first
        # determine the ax2 bounding box
        # points: a 2x2 numpy array of the form [[x0, y0], [x1, y1]]
        ax2box = ax2.get_window_extent().get_points()
        # seems to be expressed in pixels so convert to relative coordinates
#        print ax2box
        # figure size in pixels
        figsize_x_pix = plot.figsize_x*plot.dpi
        figsize_y_pix = plot.figsize_y*plot.dpi
        # ax2 box in relative coordinates
        ax2box[:,0] = ax2box[:,0] / figsize_x_pix
        ax2box[:,1] = ax2box[:,1] / figsize_y_pix
#        print ax2box[0,0], ax2box[1,0], ax2box[0,1], ax2box[1,1]
        # left position new box at 10% of x1
        left   = ax2box[0,0] + ((ax2box[1,0] - ax2box[0,0]) * 0.15)
        bottom = ax2box[0,1] + ((ax2box[1,1] - ax2box[0,1]) * 0.30)  # x2
        width  = (ax2box[1,0] - ax2box[0,0]) * 0.35
        height = (ax2box[1,1] - ax2box[0,1]) * 0.6
#        print [left, bottom, width, height]

        # left inset plot.
        # [left, bottom, width, height]
#        ax2a = plot.fig.add_axes([0.42, 0.6, .45, .25])
        ax2a = plot.fig.add_axes([left, bottom, width, height])
        ax2a.plot((w/np.pi)*nyq_rate, np.absolute(h), linewidth=2)
        ax2a.set_xlim(0,8.0)
        ax2a.set_ylim(0.9985, 1.001)
        ax2a.grid(True)

        # right inset plot
        left   = ax2box[0,0] + ((ax2box[1,0] - ax2box[0,0]) * 0.62)
        bottom = ax2box[0,1] + ((ax2box[1,1] - ax2box[0,1]) * 0.30)  # x2
        width  = (ax2box[1,0] - ax2box[0,0]) * 0.35
        height = (ax2box[1,1] - ax2box[0,1]) * 0.6

        # Lower inset plot
#        ax2b = plot.fig.add_axes([0.42, 0.25, .45, .25])
        ax2b = plot.fig.add_axes([left, bottom, width, height])
        ax2b.plot((w/np.pi)*nyq_rate, np.absolute(h), linewidth=2)
        ax2b.set_xlim(12.0, 20.0)
        ax2b.set_ylim(0.0, 0.0025)
        ax2b.grid(True)

        #------------------------------------------------
        # Plot the original and filtered signals.
        #------------------------------------------------

        # The phase delay of the filtered signal.
        delay = 0.5 * (N-1) / sample_rate

        plot_nr += 1
        ax3 = plot.fig.add_subplot(plot.nr_rows, plot.nr_cols, plot_nr)
        # Plot the original signal.
        ax3.plot(time, data, label='original signal')
        # Plot the filtered signal, shifted to compensate for the phase delay.
        ax3.plot(time-delay, filtered_x, 'r-', label='filtered signal')
        # Plot just the "good" part of the filtered signal.  The first N-1
        # samples are "corrupted" by the initial conditions.
        ax3.plot(time[N-1:]-delay, filtered_x[N-1:], 'g', linewidth=4)

        ax3.set_xlabel('t')
        ax3.grid(True)

        plot.save_fig()


    def scipy_example(self, time, data, figpath, sample_rate=None):
        """
        Example from the SciPy Cookboock, see
        http://www.scipy.org/Cookbook/FIRFilter
        """

        chk.array_1d(time)
        chk.array_1d(data)

        if not sample_rate:
            sample_rate = calc_sample_rate(time)

        #------------------------------------------------
        # Create a FIR filter and apply it to data[:,channel]
        #------------------------------------------------

        # The Nyquist rate of the signal.
        nyq_rate = sample_rate / 2.0

        # The desired width of the transition from pass to stop,
        # relative to the Nyquist rate.  We'll design the filter
        # with a 5 Hz transition width.
        width = 5.0/nyq_rate

        # The desired attenuation in the stop band, in dB.
        ripple_db = 60.0

        # Compute the order and Kaiser parameter for the FIR filter.
        N, beta = sp.signal.kaiserord(ripple_db, width)

        # The cutoff frequency of the filter.
        cutoff_hz = 10.0

        # Use firwin with a Kaiser window to create a lowpass FIR filter.
        taps = sp.signal.firwin(N, cutoff_hz/nyq_rate, window=('kaiser', beta))

        # Use lfilter to filter x with the FIR filter.
        filtered_x = sp.signal.lfilter(taps, 1.0, data)

        #------------------------------------------------
        # Setup the figure parameters
        #------------------------------------------------
        figfile = 'filterdesign'

        plot = mplutils.A4Tuned()
        plot.setup(figpath+figfile, nr_plots=3, grandtitle=figfile,
                         figsize_y=20, wsleft_cm=2.)

        #------------------------------------------------
        # Plot the FIR filter coefficients.
        #------------------------------------------------
        plot_nr = 1
        ax1 = plot.fig.add_subplot(plot.nr_rows, plot.nr_cols, plot_nr)
        ax1.plot(taps, 'bo-', linewidth=2)
        ax1.set_title('Filter Coefficients (%d taps)' % N)
        ax1.grid(True)

        #------------------------------------------------
        # Plot the magnitude response of the filter.
        #------------------------------------------------

        plot_nr += 1
        ax2 = plot.fig.add_subplot(plot.nr_rows, plot.nr_cols, plot_nr)

        w, h = sp.signal.freqz(taps, worN=8000)
        ax2.plot((w/np.pi)*nyq_rate, np.absolute(h), linewidth=2)
        ax2.set_xlabel('Frequency (Hz)')
        ax2.set_ylabel('Gain')
        ax2.set_title('Frequency Response')
        ax2.set_ylim(-0.05, 1.05)
#        ax2.grid(True)

        # in order to place the nex axes inside following figure, first
        # determine the ax2 bounding box
        # points: a 2x2 numpy array of the form [[x0, y0], [x1, y1]]
        ax2box = ax2.get_window_extent().get_points()
        # seems to be expressed in pixels so convert to relative coordinates
#        print ax2box
        # figure size in pixels
        figsize_x_pix = plot.figsize_x*plot.dpi
        figsize_y_pix = plot.figsize_y*plot.dpi
        # ax2 box in relative coordinates
        ax2box[:,0] = ax2box[:,0] / figsize_x_pix
        ax2box[:,1] = ax2box[:,1] / figsize_y_pix
#        print ax2box[0,0], ax2box[1,0], ax2box[0,1], ax2box[1,1]
        # left position new box at 10% of x1
        left   = ax2box[0,0] + ((ax2box[1,0] - ax2box[0,0]) * 0.15)
        bottom = ax2box[0,1] + ((ax2box[1,1] - ax2box[0,1]) * 0.30)  # x2
        width  = (ax2box[1,0] - ax2box[0,0]) * 0.35
        height = (ax2box[1,1] - ax2box[0,1]) * 0.6
#        print [left, bottom, width, height]

        # left inset plot.
        # [left, bottom, width, height]
#        ax2a = plot.fig.add_axes([0.42, 0.6, .45, .25])
        ax2a = plot.fig.add_axes([left, bottom, width, height])
        ax2a.plot((w/np.pi)*nyq_rate, np.absolute(h), linewidth=2)
        ax2a.set_xlim(0,8.0)
        ax2a.set_ylim(0.9985, 1.001)
        ax2a.grid(True)

        # right inset plot
        left   = ax2box[0,0] + ((ax2box[1,0] - ax2box[0,0]) * 0.62)
        bottom = ax2box[0,1] + ((ax2box[1,1] - ax2box[0,1]) * 0.30)  # x2
        width  = (ax2box[1,0] - ax2box[0,0]) * 0.35
        height = (ax2box[1,1] - ax2box[0,1]) * 0.6

        # Lower inset plot
#        ax2b = plot.fig.add_axes([0.42, 0.25, .45, .25])
        ax2b = plot.fig.add_axes([left, bottom, width, height])
        ax2b.plot((w/np.pi)*nyq_rate, np.absolute(h), linewidth=2)
        ax2b.set_xlim(12.0, 20.0)
        ax2b.set_ylim(0.0, 0.0025)
        ax2b.grid(True)

        #------------------------------------------------
        # Plot the original and filtered signals.
        #------------------------------------------------

        # The phase delay of the filtered signal.
        delay = 0.5 * (N-1) / sample_rate

        plot_nr += 1
        ax3 = plot.fig.add_subplot(plot.nr_rows, plot.nr_cols, plot_nr)
        # Plot the original signal.
        ax3.plot(time, data, label='original signal')
        # Plot the filtered signal, shifted to compensate for the phase delay.
        ax3.plot(time-delay, filtered_x, 'r-', label='filtered signal')
        # Plot just the "good" part of the filtered signal.  The first N-1
        # samples are "corrupted" by the initial conditions.
        ax3.plot(time[N-1:]-delay, filtered_x[N-1:], 'g', linewidth=4)

        ax3.set_xlabel('t')
        ax3.grid(True)

        plot.save_fig()


'''
Created on 10/01/2015

@author: mmpe
'''
import numpy as np

def low_pass(input, delta_t, tau, method=1):
    from wetb.signal.filters import cy_filters

    if isinstance(tau, (int, float)):
        return cy_filters.cy_low_pass_filter(input.astype(np.float64), delta_t, tau)
    else:
        if len(input.shape)==2:
            return cy_filters.cy_dynamic_low_pass_filter_2d(input.astype(np.float64), delta_t, tau, method)
        else:
            return cy_filters.cy_dynamic_low_pass_filter(input.astype(np.float64), delta_t, tau, method)

def high_pass(input, delta_t, tau):
    from wetb.signal.filters import cy_filters

    return cy_filters.cy_high_pass_filter(input.astype(np.float64), delta_t, tau)



from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import
from builtins import dict
from builtins import zip
from future import standard_library
standard_library.install_aliases()
from six import exec_
import time
import inspect
def get_time(f):
    """Get time decorator
    returns (return_values, time_of_execution)

    >>> @get_time
    >>> def test():
    >>>    time.sleep(1)
    >>>    return "end"
    >>>
    >>> test()
    ('end', 0.999833492421551)
    """
    def wrap(*args, **kwargs):
        t = time.time()
        res = f(*args, **kwargs)
        return res, time.time() - t
    w = wrap
    w.__name__ = f.__name__
    return w


def print_time(f):
    """Print time decorator
    prints name of method and time of execution

    >>> @print_time
    >>> def test():
    >>>    time.sleep(1)
    >>>
    >>> test()
    test            1.000s
    """
    def wrap(*args, **kwargs):
        t = time.time()
        res = f(*args, **kwargs)
        print ("%-12s\t%.3fs" % (f.__name__, time.time() - t))
        return res
    w = wrap
    w.__name__ = f.__name__
    return w


cum_time = {}
def print_cum_time(f):
    """Print cumulated time decorator
    prints name of method and cumulated time of execution

    >>> @print_cum_time
    >>> def test():
    >>>    time.sleep(1)
    >>>
    >>> test()
    test            0001 calls, 1.000000s, 1.000000s pr. call'
    >>> test()
    test            0002 calls, 2.000000s, 1.000000s pr. call'
    """
    if f not in cum_time:
        cum_time[f] = (0, 0)

    def wrap(*args, **kwargs):
        t = time.time()
        res = f(*args, **kwargs)
        ct = cum_time[f][1] + time.time() - t
        n = cum_time[f][0] + 1
        cum_time[f] = (n, ct)
        print ("%-12s\t%.4d calls, %03fs, %fs pr. call'" % (f.__name__, n, ct, ct / n))
        return res
    w = wrap
    w.__name__ = f.__name__
    return w

def print_line_time(f):
    """Execute one line at the time and prints the time of execution.
    Only for non-branching and non-looping code

    prints: time_of_line, cumulated_time, code_line


    >>> @print_line_time
    >>> def test():
    >>>    time.sleep(.3)
    >>>    time.sleep(.5)
    >>>
    >>> test()
    0.300s    0.300s    time.sleep(.3)
    0.510s    0.810s    time.sleep(.51)

    """
    def wrap(*args, **kwargs):
        arg_names, varargs, varkw, defaults = inspect.getargspec(f)
        kwargs[varargs] = args[len(arg_names):]
        kwargs[varkw] = {}
        for k, v in kwargs.items():
            if k not in tuple(arg_names) + (varargs, varkw):
                kwargs.pop(k)
                kwargs[varkw][k] = v
        if defaults:
            kwargs.update(dict(zip(arg_names[::-1], defaults[::-1])))
        kwargs.update(dict(zip(arg_names, args)))


        lines = inspect.getsourcelines(f)[0][2:]
        tcum = time.time()
        locals = kwargs
        gl = f.__globals__

        for l in lines:
            tline = time.time()
            exec(l.strip(), locals, gl)  #res = f(*args, **kwargs)
            print ("%.3fs\t%.3fs\t%s" % (time.time() - tline, time.time() - tcum, l.strip()))
    w = wrap
    w.__name__ = f.__name__
    return w




import os
import numpy as np
import pandas as pd
import itertools


"""IEC refers to IEC61400-1(2005)

Questions:
dlc1.4 (hansen: 100s, no turb), why not NTM, NWP 600s and 6 seeds 
seed numbering and reuse???
"""


class DLC():

    def __init__(self, Description, Turb, Shear, Gust, Fault=None, variables={}):
        self.Description = Description
        self.Turb = Turb
        self.Shear = Shear
        self.Gust = Gust
        self.Fault = Fault
        self.variables = variables
        self.variables.update({k.lower(): v for k, v in variables.items()})
        turb_class = self.iec_wt_class[1].lower()
        assert turb_class in 'abc'
        self.I_ref = {'a': .16, 'b': .14, 'c': .12}[turb_class]  # IEC61400-1(2005) table 1
        wind_class = int(self.iec_wt_class[0])
        assert 1 <= wind_class <= 3
        self.V_ref = {1: 50, 2: 42.5, 3: 37.5}[wind_class]

    @classmethod
    def getattr(cls, name):
        try:

            return getattr(cls, name)
        except AttributeError as e:
            d = {k.lower(): k for k in dir(cls)}
            if name.lower() in d:
                return getattr(cls, d[name.lower()])
            else:
                raise e

    def __getattr__(self, name):
        try:
            return self.__getattribute__(name)
        except AttributeError as e:
            if name in self.variables:
                return self.variables[name]
            elif name.lower() in self.variables:
                return self.variables[name.lower()]
            else:
                raise e

    def get_lst(self, x):
        if isinstance(x, pd.Series):
            x = x.iloc[0]
        if ":" in str(x):
            start, step, stop = [float(eval(v, globals(), self.variables)) for v in x.split(":")]
            return list(np.arange(start, stop + step, step))
        else:
            return [float(eval(v, globals(), self.variables)) for v in str(x).replace("/", ",").split(",")]

    @property
    def wsp_lst(self):
        return sorted(self.get_lst(self.WSP))

    @property
    def wdir_lst(self):
        return self.get_lst(self.Wdir)

    def case_arg_lst_product(self, **kwargs):
        case_arg_lst = []
        for dict_lst in itertools.product(*[m(self, **kwargs) for m in [DLC.WspWdir, self.Turb, self.Shear, self.Gust, self.Fault] if m is not None]):
            ids = {k: v for d in dict_lst for k, v in d.items() if '_id' in k}
            d = {k: v for d in dict_lst for k, v in d.items() if '_id' not in k}
            name = [self.Name, 'wsp%02d' % d['V_hub'], "wdir%03d" % (d['wdir'] % 360)]
            if 'seed_id' in ids:
                name.append("s%04d" % d['seed'])
            if 'ews_id' in ids:
                name.append("ews%s" % d['shear']['sign'])
            d['Name'] = "_".join(name)
            case_arg_lst.append(d)
        return case_arg_lst

    def to_pandas(self):
        case_dict_lst = []

        for V_hub in self.wsp_lst:
            for wdir in self.wdir_lst:
                default_kwargs = {'Folder': self.Name,
                                  'simulation_time': self.Time,
                                  'V_hub': V_hub,
                                  'wdir': wdir,
                                  }
                for case_args in self.case_arg_lst_product(**default_kwargs):
                    case_args.update({k: v for k, v in default_kwargs.items() if k not in case_args})
                    case_dict_lst.append(case_args)
        return pd.DataFrame(case_dict_lst)

    # ===============================================================================
    # General
    # ===============================================================================
    def WspWdir(self, V_hub, wdir, **_):
        return [{'V_hub': V_hub, 'wdir': wdir}]

    def NONE(self, **_):
        return [{}]

    def NaN(self, **_):
        return [{}]

    # ===============================================================================
    # Turbulence models
    # ===============================================================================
    def NoTurb(self, **_):
        return [{'seed': None}]

    def NTM(self, V_hub, **_):
        # Normal turbulence model IEC section 6.3.1.3
        s0 = int((V_hub - self.Vin) // self.Vstep * 100 + 1001)
        ti = (self.I_ref * (0.75 * V_hub + 5.6)) / V_hub  # IEC (11)
        return [{'seed_id': 's%04d' % (s),
                 'ti': ti,
                 'seed': s} for s in range(s0, s0 + int(self.Seeds))]

    def ETM(self, V_hub, **_):
        # Extreme Turbulence model
        # IEC (9)
        V_ave = .2 * self.V_ref

        # IEC (19)
        c = 2
        ti = c * self.I_ref * (0.072 * (V_ave / c + 3) * (V_hub / c - 4) + 10) / V_hub

        s0 = int((V_hub - self.Vin) // self.Vstep * 100 + 1001)

        return [{'seed_id': 's%04d' % (s),
                 'ti': ti,
                 'seed': s} for s in range(s0, s0 + int(self.Seeds))]

    # ===============================================================================
    # Shear profiles
    # ===============================================================================

    def NWP(self, **_):
        # The normal wind profile model IEC section 6.3.1.2
        return [{'shear': {'type': 'NWP', 'profile': ('power', .2)}}]

    def EWS(self, V_hub, **_):
        # Extreme wind shear, IEC section 6.3.2.6
        beta = 6.4
        T = 12
        sigma_1 = self.I_ref * (0.75 * V_hub + 5.6)  # IEC (11)

        D = self.D
        lambda_1 = 42 if self.z_hub < 60 else .7 * self.z_hub
        A = (2.5 + 0.2 * beta * sigma_1 * (D / lambda_1)**0.25) / D  # IEC (26) & (27)

        return [{'shear': {'type': 'EWS', 'profile': ('power', .2), 'A': A, 'T': T, 'sign': ews_sign},
                 'ews_id': ews_sign}
                for ews_sign in ['++', '+-', '-+', '--']]

    # ===========================================================================
    # Gusts
    # ===========================================================================

    def ECD(self, V_hub, **_):
        # Extreme coherrent gust with direction change IEC 6.3.2.5
        # IEC section 6.3.2.5
        # IEC (22)
        V_cg = 15  # magnitude of gust
        # IEC (24)
        theta_cg = (720 / V_hub, 180)[V_hub < 4]  # direction change
        T = 10  # rise time
        return [{'Gust': {'type': 'ECD', 'V_cg': V_cg, 'theta_cg': theta_cg, 'T': T}}]

    # ===============================================================================
    # Faults
    # ===============================================================================

    def GridLoss10(self, **_):
        return [{'Fault': {'type': 'GridLoss', 'T': 10}}]


class DLB():
    def __init__(self, dlc_definition_dict_lst, variables):
        self.dlc_definition_dict_lst = dlc_definition_dict_lst
        self.variables = variables

        def isNone(v):
            return str(v).lower().strip() in ['', 'none', 'nan']

        def merge_dict(d1, d2):
            d = d1.copy()
            d.update(d2)
            return d

        self.dlc_dict = {}
        self.dataframe_dict = {}

        for dlc_dict in dlc_definition_dict_lst:
            name = dlc_dict['Name']
            if isNone(name):
                continue

            kwargs = {k: DLC.getattr(str(dlc_dict[k]))
                      for k in ['Turb', 'Shear', 'Gust', 'Fault']}
            kwargs['Description'] = dlc_dict['Description']
            variables.update(dlc_dict)
            dlc = DLC(variables=variables, **kwargs)
            self.dataframe_dict[name] = dlc.to_pandas()

    def get_overview(self):
        cols = ['Name', 'Description', 'WSP', 'Wdir', 'Turb', 'Seeds', 'Shear', 'Gust', 'Fault', 'Time']
        df_dlc = pd.DataFrame(
            [{k: dlc_def[k] for k in cols} for dlc_def in self.dlc_definition_dict_lst], columns=cols)
        variables = [('Vin', 'Cut-in wind speed'),
                     ('Vout', 'Cut-out wind speed'),
                     ('Vr', 'Rated wind speed'),
                     ('D', 'Rotor diameter'),
                     ('z_hub', 'Hub height'),
                     ('Vstep', 'Wind speed distribution step'),
                     ('iec_wt_class', 'IEC wind turbine class, e.g. 1A')]
        df_vars = pd.DataFrame([{'Name': n, 'Value': self.variables[n], 'Description': d}
                                for n, d in variables], columns=['Name', 'Value', 'Description'])
        return {'DLC': df_dlc, 'Variables': df_vars}

    def to_excel_overview(self, filename):
        if os.path.dirname(filename) != "":
            os.makedirs(os.path.dirname(filename), exist_ok=True)
        writer = pd.ExcelWriter(filename)
        for k, v in self.get_overview().items():
            v.to_excel(writer, k, index=False)
        writer.save()

    @staticmethod
    def from_excel(filename):
        df_vars = pd.read_excel(filename, sheet_name='Variables',
                                index_col='Name')
        df_vars.fillna('', inplace=True)
        variables = {name: value for name, value in zip(df_vars.index, df_vars.Value.values)}

        dlb_def = pd.read_excel(filename, 'DLC', skiprows=[1], )
        dlb_def.columns = [c.strip() for c in dlb_def.columns]
        return DLB([row for _, row in dlb_def.iterrows() if row['Name'] is not np.nan], variables)

    def __getitem__(self, key):
        return self.dataframe_dict[key]

    def to_pandas(self):
        return pd.Panel(self.dataframe_dict)

    def to_excel(self, filename):
        if os.path.dirname(filename) != "":
            os.makedirs(os.path.dirname(filename), exist_ok=True)
        writer = pd.ExcelWriter(filename)
        for k, v in self.dataframe_dict.items():
            v.to_excel(writer, k, index=False)
        writer.save()


class DTU_IEC64100_1_Ref_DLB(DLB):
    def __init__(self, iec_wt_class, Vin, Vout, Vr, D, z_hub):
        """
        NOTE!!!!!!!!!!!
        SEVERAL DLCS ARE MISSING
        """
        Vstep = 2
        Name, Description, WSP, Wdir, Time = 'Name', 'Description', 'WSP', 'Wdir', 'Time'
        Turb, Seeds, Shear, Gust, Fault = 'Turb', 'Seeds', 'Shear', 'Gust', 'Fault'

        dlc_definition_dict_lst = [
            {Name: 'DLC12', Description: 'Normal production', WSP: 'Vin:2:Vout', Wdir: '-10/0/10',
                Turb: 'NTM', Seeds: 6, Shear: 'NWP', Gust: None, Fault: None, Time: 600},
            {Name: 'DLC13', Description: 'Normal production with high turbulence', WSP: 'Vin:2:Vout', Wdir: '-10/0/10',
                Turb: 'ETM', Seeds: 6, Shear: 'NWP', Gust: None, Fault: None, Time: 600},
            {Name: 'DLC14', Description: 'Normal production with gust and direction change', WSP: 'Vr/Vr+2/Vr-2', Wdir: 0,
                Turb: 'NoTurb', Seeds: None, Shear: 'NWP', Gust: 'ECD', Fault: None, Time: 100},
            {Name: 'DLC15', Description: 'Normal production with extreme wind shear', WSP: 'Vin:2:Vout', Wdir: 0,
                Turb: 'NoTurb', Seeds: None, Shear: 'EWS', Gust: None, Fault: None, Time: 100},
            {Name: 'DLC21', Description: 'Loss of electical network', WSP: 'Vin:2:Vout', Wdir: '-10/0/10',
                Turb: 'NTM', Seeds: 4, Shear: 'NWP', Gust: None, Fault: 'GridLoss10', Time: 100},
            {Name: 'DLC22y', Description: 'Abnormal yaw error', WSP: 'Vin:2:Vout', Wdir: '15:15:345',
                Turb: 'NTM', Seeds: 1, Shear: 'NWP', Gust: None, Fault: None, Time: 600}

        ]
        variables = {'iec_wt_class': iec_wt_class, 'Vin': Vin,
                     'Vout': Vout, 'Vr': Vr, 'D': D, 'z_hub': z_hub, 'Vstep': Vstep}
        DLB.__init__(self, dlc_definition_dict_lst, variables)


def main():
    if __name__ == '__main__':

        dlb = DTU_IEC64100_1_Ref_DLB('1A', 4, 10, 8, 180, 110)
        print(dlb.get_overview())
        print(dlb['DLC15'])

        # save and load excel overview
        dlb.to_excel_overview('overview.xlsx')
        dlb = DLB.from_excel('overview.xlsx')

        # save all cases to excel
        dlb.to_excel('cases.xlsx')

        # Save generate hawc2 input files
        from wetb.hawc2.tests import test_files
        from wetb.dlb.hawc2_iec_dlc_writer import HAWC2_IEC_DLC_Writer
        path = os.path.dirname(test_files.__file__) + '/simulation_setup/DTU10MWRef6.0/'
        writer = HAWC2_IEC_DLC_Writer(path + 'htc/DTU_10MW_RWT.htc', diameter=180)
        # set write_input_files to True to write the files
        writer.from_pandas(dlb, write_input_files=False)


main()



import pandas as pd
from wetb.hawc2.tests import test_files
from wetb.hawc2.hawc2_input_writer import HAWC2InputWriter
import os
import shutil
from wetb.hawc2.htc_file import HTCFile
import pytest
from wetb.hawc2 import hawc2_input_writer
from tests.test_examples import run_module_main

path = os.path.dirname(test_files.__file__) + '/simulation_setup/DTU10MWRef6.0/'


def clean_htctmp():
    if os.path.isdir(path + 'htc/tmp'):
        shutil.rmtree(path + 'htc/tmp')


@pytest.yield_fixture(autouse=True)
def run_around_tests():
    clean_htctmp()
    yield
    clean_htctmp()


@pytest.fixture
def h2writer():
    htc_base_file = path + 'htc/DTU_10MW_RWT.htc'
    return HAWC2InputWriter(htc_base_file)


def test_pandas2htc(h2writer):

    wsp_lst = [4, 6, 8]
    df = pd.DataFrame({'wind.wsp': wsp_lst, 'Name': ['c1', 'c2', 'c3'], 'Folder': ['tmp', 'tmp', 'tmp']})
    h2writer.from_pandas(df)
    for i, wsp in enumerate(wsp_lst, 1):
        htc = HTCFile(path + "htc/tmp/c%d.htc" % i)
        assert htc.wind.wsp[0] == wsp


def test_excel2htc(h2writer):
    h2writer.from_excel(os.path.dirname(test_files.__file__) + "/htc_input_table.xlsx")
    for i, wsp in enumerate([4, 6], 1):
        htc = HTCFile(path + "htc/tmp/a%d.htc" % i)
        assert htc.wind.wsp[0] == wsp


def test_hawc2_writer_main():
    run_module_main(hawc2_input_writer)


def test_hawc2_writer():
    path = os.path.dirname(test_files.__file__) + '/simulation_setup/DTU10MWRef6.0/'
    htc_base_file = path + 'htc/DTU_10MW_RWT.htc'

    # HAWC2InputWriter
    class MyWriter(HAWC2InputWriter):
        def set_time(self, htc, time, **_):
            htc.set_time(self.time_start, self.time_start + time)

    myWriter = MyWriter(htc_base_file, time_start=100)
    ps = myWriter("w1", 'tmp', time=600, **{"wind.wsp": 10})
    htc = HTCFile(path + "htc/tmp/w1.htc")
    assert htc.simulation.time_stop[0] == 700
    assert htc.wind.wsp[0] == 10
    assert (ps.Name == 'w1')


def test_CVF2pandas(h2writer):
    constants = {'simulation.time_stop': 100}
    variables = {'wind.wsp': [4, 6, 8],
                 'wind.tint': [0.1, 0.15, 0.2]}
    functions = {'Name': lambda x: 'sim_wsp' + str(x['wind.wsp']) + '_ti' + str(x['wind.tint'])}

    df = h2writer.from_CVF(constants, variables, functions)
    assert len(df) == 9
    assert set(list(df)) == set(['simulation.time_stop', 'wind.wsp', 'wind.tint', 'Name'])


def test_from_definition(h2writer):
    from wetb.hawc2.tests.test_files import dlc_definition

    definition_file = dlc_definition.__file__
    df = h2writer.from_definition(definition_file)
    assert len(df) == 9
    assert set(list(df)) == set(['simulation.time_stop', 'wind.wsp', 'wind.tint', 'Name'])



# -*- coding: utf-8 -*-




constants = {'simulation.time_stop':100}


variables = {'wind.wsp': [4, 6, 8],
             'wind.tint': [0.1, 0.15, 0.2]}



functions = {'Name': lambda x: 'sim_wsp' + str(x['wind.wsp']) + '_ti' + str(x['wind.tint'])}






# -*- coding: utf-8 -*-
"""
Created on Thu Aug 04 11:09:43 2016

@author: tlbl
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
import unittest

from wetb.control import control
import numpy as np


class TestControl(unittest.TestCase):

    def setUp(self):
        dQdt = np.array([-0.126876918E+03, -0.160463547E+03, -0.211699586E+03,
                         -0.277209984E+03, -0.351476131E+03, -0.409411354E+03,
                         -0.427060299E+03, -0.608498644E+03, -0.719141594E+03,
                         -0.814129630E+03, -0.899248007E+03, -0.981457622E+03,
                         -0.106667910E+04, -0.114961807E+04, -0.123323210E+04,
                         -0.131169210E+04, -0.138758236E+04, -0.145930419E+04,
                         -0.153029102E+04, -0.159737975E+04, -0.166846850E+04])

        pitch = np.array([0.2510780000E+00, 0.5350000000E-03, 0.535000000E-03,
                          0.5350000000E-03, 0.5350000000E-03, 0.535000000E-03,
                          0.5350000000E-03, 0.3751976000E+01, 0.625255700E+01,
                          0.8195032000E+01, 0.9857780000E+01, 0.113476710E+02,
                          0.1271615400E+02, 0.1399768300E+02, 0.152324310E+02,
                          0.1642177100E+02, 0.1755302300E+02, 0.186442750E+02,
                          0.1970333100E+02, 0.2073358600E+02, 0.217410280E+02])

        I = np.array([0.4394996114E+08, 0.4395272885E+08, 0.4395488725E+08,
                      0.4395301987E+08, 0.4394561932E+08, 0.4393327166E+08,
                      0.4391779133E+08, 0.4394706335E+08, 0.4395826989E+08,
                      0.4396263773E+08, 0.4396412693E+08, 0.4396397777E+08,
                      0.4396275304E+08, 0.4396076315E+08, 0.4395824699E+08,
                      0.4395531228E+08, 0.4395201145E+08, 0.4394837798E+08,
                      0.4394456127E+08, 0.4394060604E+08, 0.4393647769E+08])

        self.dQdt = dQdt
        self.pitch = pitch
        self.I = I

    def test_pitch_controller_tuning(self):

        crt = control.Control()
        P = 0.
        Omr = 12.1
        om = 0.10
        csi = 0.7
        i = 5
        kp, ki, K1, K2 = crt.pitch_controller_tuning(self.pitch[i:],
                                                     self.I[i:],
                                                     self.dQdt[i:],
                                                     P, Omr, om, csi)

        self.assertAlmostEqual(kp, 1.596090243644432, places=10)
        self.assertAlmostEqual(ki, 0.71632362627138424, places=10)
        self.assertAlmostEqual(K1, 10.01111637532056, places=10)
        self.assertAlmostEqual(K2, 599.53659803157643, places=10)

    def test_regions(self):

        crt = control.Control()

        pitch = np.array([0.,-2.,-2.,-2.,-2.,-2.,-2.,-1., 0., ])
        omega = np.array([1., 1., 1., 2., 3., 3., 3., 3., 3., ])
        power = np.array([1., 2., 3., 4., 5., 6., 7., 7., 7., ])

        istart, iend = 0, -1
        i1, i2, i3 = crt.select_regions(pitch[istart:iend], omega[istart:iend],
                                        power[istart:iend])
        self.assertEqual(i1, 2)
        self.assertEqual(i2, 4)
        self.assertEqual(i3, 6)

        istart, iend = 3, -1
        i1, i2, i3 = crt.select_regions(pitch[istart:iend], omega[istart:iend],
                                        power[istart:iend])
        self.assertEqual(i1, 0)
        self.assertEqual(i2, 1)
        self.assertEqual(i3, 3)

        istart, iend = 5, -1
        i1, i2, i3 = crt.select_regions(pitch[istart:iend], omega[istart:iend],
                                        power[istart:iend])
        self.assertEqual(i1, 0)
        self.assertEqual(i2, 0)
        self.assertEqual(i3, 1)

        istart, iend = 6, -1
        i1, i2, i3 = crt.select_regions(pitch[istart:iend], omega[istart:iend],
                                        power[istart:iend])
        self.assertEqual(i1, 0)
        self.assertEqual(i2, 0)
        self.assertEqual(i3, 0)

        istart, iend = 5, -2
        i1, i2, i3 = crt.select_regions(pitch[istart:iend], omega[istart:iend],
                                        power[istart:iend])
        self.assertEqual(i1, 0)
        self.assertEqual(i2, 0)
        self.assertEqual(i3, 1)

        istart, iend = 3, -3
        i1, i2, i3 = crt.select_regions(pitch[istart:iend], omega[istart:iend],
                                        power[istart:iend])
        self.assertEqual(i1, 0)
        self.assertEqual(i2, 1)
        self.assertEqual(i3, 2)

        istart, iend = 2, -4
        i1, i2, i3 = crt.select_regions(pitch[istart:iend], omega[istart:iend],
                                        power[istart:iend])
        self.assertEqual(i1, 0)
        self.assertEqual(i2, 2)
        self.assertEqual(i3, 2)

        istart, iend = 0, 3
        i1, i2, i3 = crt.select_regions(pitch[istart:iend], omega[istart:iend],
                                        power[istart:iend])
        self.assertEqual(i1, 0)
        self.assertEqual(i2, 0)
        self.assertEqual(i3, 2)

if __name__ == "__main__":

    unittest.main()



from wetb.utils.cluster_tools.pbsfile import PBSFile, Template, PBSMultiRunner
from wetb.utils.cluster_tools import ssh_client
from wetb.utils.cluster_tools.ssh_client import SSHClient
import io
from wetb.utils.cluster_tools.pbsjob import SSHPBSJob, DONE
import time
import pytest
try:
    import x
except ImportError:
    x = None


def test_template():
    t = Template('[a]B[c]')
    assert t(a="A", c="C") == "ABC"
    assert t(a="[c]", c="C") == "CBC", "%s!=%s" % (t(a="[c]", c="C"), 'CBC')


def test_pbs_file_str():
    pbsfile = PBSFile('/home/user/tmp', "test", '''python -c "print('hello world')"''', 'workq')
    ref = """### Jobid
#PBS -N test
### Standard Output
#PBS -o /home/user/tmp/./stdout/test.out
### merge stderr into stdout
#PBS -j oe
#PBS -W umask=0003
### Maximum wallclock time format HOURS:MINUTES:SECONDS
#PBS -l walltime=00:10:00
#PBS -l nodes=1:ppn=1
### Queue name
#PBS -q workq
cd /home/user/tmp
mkdir -p stdout
if [ -z "$PBS_JOBID" ]; then echo "Run using qsub"; exit ; fi
pwd
python -c "print('hello world')"
exit
"""
    assert str(pbsfile) == ref


def test_pbs_file():
    if x is None:
        pytest.xfail("Password missing")
    pbsfile = PBSFile("/home/mmpe/tmp", "test", '''python -c "print('hello world')"''', 'workq')
    ssh = SSHClient("jess.dtu.dk", 'mmpe', x.mmpe)
    pbs_job = SSHPBSJob(ssh)
    pbs_job.submit(pbsfile, "./tmp")
    with pbs_job.ssh:
        start = time.time()
        while time.time() < start + 10:
            time.sleep(.1)
            if pbs_job.status == DONE:
                break
        else:
            raise Exception("job not finished within 10 s")
        _, out, _ = ssh.execute('cat ./tmp/stdout/test.out')
    assert "hello world" in out


@pytest.mark.parametrize('i,s', [("01:02:03", "01:02:03"),
                                 (5, "00:00:05"),
                                 (4000, '01:06:40')])
def test_pbs_walltime(i, s):
    pbsfile = PBSFile("./tmp", "test", '', 'workq', walltime=i)
    assert pbsfile.walltime == s


def test_pbs_multirunner():
    pbs = PBSMultiRunner("/home/user/tmp", )
    ref = r"""### Jobid
#PBS -N pbs_multirunner
### Standard Output
#PBS -o /home/user/tmp/./stdout/pbs_multirunner.out
### merge stderr into stdout
#PBS -j oe
#PBS -W umask=0003
### Maximum wallclock time format HOURS:MINUTES:SECONDS
#PBS -l walltime=01:00:00
#PBS -l nodes=1:ppn=1
### Queue name
#PBS -q workq
cd /home/user/tmp
mkdir -p stdout
if [ -z "$PBS_JOBID" ]; then echo "Run using qsub"; exit ; fi
pwd
echo "import os
import glob
import numpy as np
import re

# find available nodes
with open(os.environ['PBS_NODEFILE']) as fid:
    files = set([f.strip() for f in fid.readlines() if f.strip() != ''])
pbs_files = [os.path.join(root, f) for root, folders, f_lst in os.walk('.') for f in f_lst if f.endswith('.in')]

# Make a list of [(pbs_in_filename, stdout_filename, walltime),...]
pat = re.compile(r'[\s\S]*#\s*PBS\s+-o\s+(.*)[\s\S]*(\d\d:\d\d:\d\d)[\s\S]*')

def get_info(f):
    try:
        with open(f) as fid:
            return (f,) + pat.match(fid.read()).groups()
    except Exception:
        return (f, f.replace('.in', '.out'), '00:30:00')
pbs_info_lst = map(get_info, pbs_files)

# sort wrt walltime
pbs_info_lst = sorted(pbs_info_lst, key=lambda fow: tuple(map(int, fow[2].split(':'))))[::-1]
# make dict {node1: pbs_info_lst1, ...} and save
d = dict([(f, pbs_info_lst[i::len(files)]) for i, f in enumerate(files)])
with open('pbs.dict', 'w') as fid:
    fid.write(str(d))

" | python

for node in `cat $PBS_NODEFILE | sort | uniq`
do
     
     ssh -T $node << EOF &
cd /home/user/tmp
python -c "import os
import multiprocessing
import platform
import time
with open('pbs.dict') as fid:
    pbs_info_lst = eval(fid.read())[platform.node()]
arg_lst = ['echo starting %s && mkdir -p %s && env PBS_JOBID=$PBS_JOBID %s &> %s && echo finished %s' %
           (f, os.path.dirname(o), f, o, f) for f, o, _ in pbs_info_lst]
print(arg_lst[0])
print('Starting %d jobs on %s' % (len(arg_lst), platform.node()))
pool = multiprocessing.Pool(int('$PBS_NUM_PPN'))
res = pool.map_async(os.system, arg_lst)
t = time.time()
for (f, _, _), r in zip(pbs_info_lst, res.get()):
    print('%-50s\t%s' % (f, ('Errorcode %d' % r, 'Done')[r == 0]))
print('Done %d jobs on %s in %ds' % (len(arg_lst), platform.node(), time.time() - t))

"
EOF
done
wait

exit
"""
    assert str(pbs) == ref



'''
Created on 18/07/2016

@author: MMPE
'''
import os
import unittest

import numpy as np
from wetb import gtsdf
from wetb.signal.subset_mean import time_trigger, subset_mean, \
    non_nan_index_trigger, revolution_trigger_old, revolution_trigger
from wetb.utils.geometry import rpm2rads


tfp = os.path.join(os.path.dirname(__file__), 'test_files/')
class TestSubsetMean(unittest.TestCase):
    def test_time_trigger(self):
        time = np.arange(0, 99.5, .5)
        np.testing.assert_array_equal(time[time_trigger(time, 20)], [0, 20, 40, 60, 80])
        np.testing.assert_array_equal(time[time_trigger(time + .5, 20)], [0, 20, 40, 60, 80])
        np.testing.assert_array_equal(time[time_trigger(time + 100000000.5, 20)], [0, 20, 40, 60, 80])
        np.testing.assert_array_equal(time[time_trigger(time, 20, 20, 60)], [20, 40, 60])
        np.testing.assert_array_equal(time_trigger(np.arange(101), 20), [0, 20, 40, 60, 80, 100])
        time, data, info = gtsdf.load(tfp + "subset_mean_test.hdf5")
        np.testing.assert_array_equal(time_trigger(time, 200), [0, 5000, 10000, 15000])

    def test_subset_mean(self):

        time, data, info = gtsdf.load(tfp + "subset_mean_test.hdf5")
        triggers = time_trigger(time, 100)
        t, p = subset_mean([time, data[:, 0]], triggers).T
        self.assertEqual(t[1], time[2500:5000].mean())
        self.assertEqual(p[1], data[2500:5000, 0].mean())

        triggers[1] = 2501
        t, p = subset_mean([time, data[:, 0]], triggers).T
        self.assertEqual(t[1], time[2501:5000].mean())
        self.assertEqual(p[1], data[2501:5000, 0].mean())

    def test_non_nan_index_trigger(self):
        sensor = np.arange(18).astype(np.float)
        sensor[[5, 11]] = np.nan
        triggers = non_nan_index_trigger(sensor, 3)
        for i1, i2 in triggers:
            self.assertFalse(np.any(np.isnan(sensor[i1:i2])))
        self.assertEqual(len(triggers), 4)


    def test_subset_mean_trigger_tuple(self):
        sensor = np.arange(18).astype(np.float)
        triggers = non_nan_index_trigger(sensor, 3)
        np.testing.assert_array_equal(subset_mean(sensor, triggers), [ 1., 4., 7., 10., 13., 16])

        #start with nan eq step, eq len
        sensor[0] = np.nan
        triggers = non_nan_index_trigger(sensor, 3)
        np.testing.assert_array_equal(subset_mean(sensor, triggers), [ 2, 5, 8, 11, 14])

        #nan in the middle, noneq step and len
        sensor = np.arange(18).astype(np.float)
        sensor[[5, 11]] = np.nan
        triggers = non_nan_index_trigger(sensor, 3)

        np.testing.assert_array_equal(subset_mean(sensor, triggers), [1, 7, 13, 16])

    def test_cycle_trigger(self):
        ds = gtsdf.Dataset(tfp+'azi.hdf5')
        azi, rpm, time = [ds(x)[8403:8803] for x in ['azi','Rot_cor','Time']]
        
        trigger = revolution_trigger_old(azi)
        np.testing.assert_array_equal(trigger, [ 17, 128, 241, 354])
        azi[64] = 358
        trigger = revolution_trigger_old(azi, (ds('Rot_cor'), np.diff(time).mean()))
        
#         import matplotlib.pyplot as plt
#         t = np.arange(len(azi))
#         plt.plot(t, azi)
#         for i1,i2 in trigger:
#             plt.plot(t[i1:i2],azi[i1:i2],'.--')
#         plt.show()
        np.testing.assert_array_equal(trigger, [ (128,241),(241,354)])


    def test_revolution_trigger(self):
        rotor_position = np.arange(0.,360*10,4)
        rotor_position += np.random.random(len(rotor_position))
        rotor_position = rotor_position % 360
        if 0:
            x1 = np.random.randint(0, len(rotor_position),10)
            print (list(x1))
            x2 = np.random.randint(0, len(rotor_position),10)
            print (list(x2))
        else:
            x1 = [447, 854,   847, 488, 412, 199, 675]
            x2 = [92, 647, 821, 422, 33, 159, 369, 99, 157, 464]
            
        rotor_position[x1] += 360
        rotor_position[x2] -= 360
        rotor_position[90] = 180
        rotor_position[270:276] = 15
        rotor_position[598:602] = [360,360,0,0]
        rotor_position[748:752] = [360,360,0,0]
        
        indexes = revolution_trigger(rotor_position, 20,1/(90/20/60))
        if 0:
            import matplotlib.pyplot as plt
            plt.plot(rotor_position)
            plt.plot(indexes, np.zeros_like(indexes),'.')
            plt.show()
        np.testing.assert_array_equal(indexes, [ 91, 180, 270, 360, 450, 540, 630, 720, 810])
        
    def test_revolution_trigger_max_rev_diff(self):
        rotor_position = np.arange(0.,360*10,4)
        rotor_position += np.random.random(len(rotor_position))
        rotor_position = rotor_position % 360
        rotor_speed = 1/(90/20/60)
        revolution_trigger(rotor_position, 20, rotor_speed*1.02, max_rev_diff=3)
        self.assertRaisesRegex(AssertionError, "No of revolution mismatch", revolution_trigger, rotor_position, 20, rotor_speed*1.02, max_rev_diff=1)
        
        
if __name__ == "__main__":
    unittest.main()
